#!/usr/bin/env python
"""
Inserts exported dashbaord json file in a given directory 
as data into pulse_imports reference table of a given qradar instance
"""

import os
import sys
import getopt
import getpass
import glob
import json
import hashlib
import requests
import time
import io
from jsonschema import validate
from requests.auth import HTTPBasicAuth
from collections import OrderedDict
requests.packages.urllib3.disable_warnings()

def usage():
    print('Usage: putrefdata.py -u <username> -d <directory> [-q <IP address>] [-p <password>] [-c] [-x <proxy>]')
    print(' -u, QRadar username (must have admin role)')
    print(' -d, directory containing dashboard templates')
    print(' -q, QRadar IP address, defaults to 127.0.0.1')
    print(' -p, QRadar user password (will be prompted if not provided)')
    print(' -c, clears the reference table before inserting')
    print(' -x, proxy to use for http requests, e.g. socks5h://localhost:1080')


proxies = None
dir_path = os.path.dirname(os.path.realpath(__file__))

def main(arg):
    global proxies
    username = ''
    directory = ''
    hostname = '127.0.0.1'
    clear = False
    pswd = '' 
    proxy = None
    try:
        opts, args = getopt.getopt(arg, "hu:p:d:q:cx:")
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            usage()
            sys.exit(2)
        elif opt == '-u':
            username = arg
        elif opt == '-p':
            pswd = arg
        elif opt == '-d':
            directory = arg
            if not os.path.exists(directory):
                print("Directory does not exist.")
                sys.exit(2)
        elif opt == '-q':
            hostname = arg
        elif opt == '-c':
            clear = True
        elif opt == '-x':
            proxy = arg
    
    if proxy is not None:
        proxies = {
          'http': proxy,
          'https': proxy
        }
    
    if not directory: 
        print("directory is required.")
        usage()
        sys.exit(2)
    if not username:
        print("username is required.")
        usage()
        sys.exit(2)

    if not pswd:
        pswd = getpass.getpass('Password:')
    
    auth = {
        "username": username,
        "password": pswd
    }
    
    try: 
      if clear:
          deleteTable(hostname, auth)
      
      createPulseImportsTableIfNotExists(hostname, auth)
      readDirectory(directory, auth, hostname)
    except requests.exceptions.RequestException as e:
        print('There was a problem connecting to QRadar at IP address "{}". Make sure you entered the correct IP address using -q option.'.format(hostname))
        print(getattr(e, 'message', repr(e)))
        sys.exit(1)

    sys.exit(0)

def checkTerminatingErrors(response):
    """ 
    Checks if script needs to be stopped based on the rest response
    and terminates the script if needed. e.g. authorization errors
    """
    responseJson = response.json()
    message = 'Unauthorized. Error: {}'
    if response.status_code == 401:
        if responseJson['code'] == 22:
            print('Invalid username or password.')
        else: 
            print(message.format(responseJson['message']))
        sys.exit(1)    
    elif response.status_code == 403:
        if responseJson['code'] == 26:
            print('The user doesn\'t have sufficient capabilities to insert data to reference table')
        else:   
            print(message.format(responseJson['message']))
        sys.exit(1)       
         
     

def deleteTable(hostname, auth):
    global proxies
    path = 'https://' + hostname + '/api/reference_data/tables/pulse_imports' + '?purge_only=true'
    header = {
        'Accept': 'application/json'
    }
    response = requests.delete(
        path,
        auth=HTTPBasicAuth(
            auth['username'],
            auth['password']),
        verify=False,
        headers=header,
        proxies=proxies
        )
    checkTerminatingErrors(response)
    responseJson = response.json()
    # repsonse puts the deletion task into a queue
    if response.status_code == 404 and responseJson['code'] == 1002:
        print('No need for clean up. pulse_imports reference table doesn\'t exist')
        return

    if 'status' in responseJson:
        verifyPath = 'https://' + hostname + \
            '/api/reference_data/tables_delete_tasks/' + str(responseJson['id'])
        while (responseJson['status'] != 'COMPLETED'):
            # wait for task to be completed before verifying
            time.sleep(0.5)
            # check if deletion task has completed
            response = requests.get(
                verifyPath,
                auth=HTTPBasicAuth(
                    auth['username'],
                    auth['password']),
                verify=False,
                headers=header,
                proxies=proxies
                )
            
            responseJson = response.json()
            if response.status_code != 200:
                print('Failed to query the status of delete reference table request. HTTP Status code: {}. Message: {}.'.format(response.status_code, responseJson['message']))
                sys.exit(1)     

        print(responseJson['message'])
    else:
        print(responseJson['message'])
        print('Unable to cleanup reference table, exiting without inserting.')
        sys.exit(2)


schema = ''


def validateJSON(data):
    global schema
    if schema == '':
        with open(os.path.join(dir_path, 'schema.json')) as f:
            schema = json.loads(f.read())
    validate(data, schema)

def createPulseImportsTableIfNotExists(hostname, auth): 
    global proxies
    # first create pulse_imports reference table
    checkPath = 'https://' + hostname + '/api/reference_data/tables/pulse_imports'
    #no need to pull everything to check for existence of the table, just request the name field
    getParams = {
        'fields': 'name'
    }
    header = {
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }
    getResponse = requests.get(
        checkPath,
        auth=HTTPBasicAuth(
            auth['username'],
            auth['password']),
        params=getParams,
        verify=False,
        headers=header,
        proxies=proxies
        )
    checkTerminatingErrors(getResponse)
    if getResponse.status_code == 404:
        print('Reference table not found, creating new pulse_imports reference table ... ')
        createURL = 'https://' + hostname + '/api/reference_data/tables'
        params = {
            'element_type': 'ALN',
            'name': 'pulse_imports'
        }
        createResponse = requests.post(
            createURL,
            auth=HTTPBasicAuth(
                auth['username'],
                auth['password']),
            params=params,
            verify=False,
            headers=header,
            proxies=proxies
            )
        checkTerminatingErrors(createResponse)
        if createResponse.status_code >= 400:  
            msg = 'Unable to create pulse_imports table. {}'.format(createResponse.json()['message'])
            print(msg)
            sys.exit(1)
          

def readDirectory(directory, auth, hostname):
    fileType = directory + "/*.json"
    fileNames = glob.glob(fileType)
    numInserted = 0
    numFailed = 0
    print('Found {} json files in the directory.'.format(len(fileNames)))
    for i in fileNames:
        with io.open(i, encoding='utf-8') as f:
            basename = os.path.basename(i)
            print('Validating "{}" ...'.format(basename))
            try:
                data = json.load(f, object_pairs_hook=OrderedDict)
            except Exception as e:
                print("Error: Not a valid json. Skipping. Reason: {}".format(getattr(e, 'message', repr(e))))
                numFailed += 1
                continue
            try: 
                validateJSON(data)
            except Exception as e:
                pathstr = '/' + '/'.join(str(p) for p in getattr(e, 'path', []))
                print("Error: Not a valid dashboard template. Skipping. Validation failed at JSON Path:'{}', Reason: {}. ".format(pathstr,getattr(e, 'message', repr(e))))
                numFailed += 1
                continue
            uuid = data['dashboards']['list'][0]['uuid']
            uuid = uuid.replace('"', '')
           
            print('Inserting "{}" into reference table ...'.format(basename))
            success = post(data, uuid, auth, hostname)
            if success:
                numInserted += 1
            else:
                numFailed += 1
    
    print("End of files in directory.")
    print("Failed:{}, Inserted:{}".format(numFailed, numInserted))


def post(data, uuid, auth, hostname):
    global proxies
    limit = 2000
    i = 0
    dashboard = {}
    #when calculating the hash use the bytes format of the data encoded as utf8
    plainData = json.dumps(data, separators=(',', ':'), ensure_ascii=False).encode('utf-8')
    uuidInsert = str("pulse-" + uuid)
    dashboard[uuidInsert] = {"SHA256": hashlib.sha256(plainData).hexdigest()}
    
    #when diving to chunks use the string format
    plainData = json.dumps(data, separators=(',', ':'))
    # create partitions of 2000 bytes
    while ((i * limit) < len(plainData)):
        start = i * limit
        end = start + limit
        chunkName = "part-" + str(i)
        chunk = plainData[start:end]
        dashboard[uuidInsert][chunkName] = chunk
        i += 1
        
    dashboard[uuidInsert]['numOfParts'] = i
    postURL = 'https://' + hostname + \
        '/api/reference_data/tables/bulk_load/pulse_imports'
    header = {
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # insert content into the reference table
    postResponse = requests.post(
        postURL, auth=HTTPBasicAuth(auth['username'], auth['password']),
        verify=False,
        headers=header,
        json=dashboard,
        proxies=proxies
        )
    checkTerminatingErrors(postResponse)
    if postResponse.status_code == 200:
        print("Succesfully inserted dashboard template into pulse_imports table.")
        return True
    else:
        print("Error: Unable to insert dashboard template into pulse_imports table. {}".format(postResponse.json()['message']))
        return False


if __name__ == "__main__":
    main(sys.argv[1:])
