import os
import glob
import json
import requests
from requests.auth import HTTPBasicAuth
import yaml
import pytest
import pexpect
import time
requests.packages.urllib3.disable_warnings()

testdir = os.path.dirname(os.path.realpath(__file__))
toolFullPath = os.path.join(testdir, '..', 'putrefdata.py')

def read_config():
    with open(os.path.join(testdir, 'config.yaml'), 'r') as stream:
        try:
            yam = yaml.load(stream)
            config = yam['QRadar']
            return config
        except yaml.YAMLError as exc:
            print(exc)


testConfig = read_config() 

def getProxies(): 
    global testConfig
    if testConfig['proxy'] is not None:
        proxies = {
            'http': testConfig['proxy'],
            'https': testConfig['proxy']
        }
        return proxies
    return None

def get_app_ID(hostname, auth):
    url = 'https://' + hostname + "/api/gui_app_framework/applications"
    resp = requests.get(
        url,
        auth=HTTPBasicAuth(
            auth['username'],
            auth['password']),
        verify=False,
        proxies=getProxies())
    assert(resp.status_code == 200)

    json_response = json.loads(resp.text)
    for application in json_response:
        if application['manifest']['name'] == "pulse.full_name":
            return (str(application['manifest']['app_id']))
    
def deleteTable(config):
    path = 'https://{}/api/reference_data/tables/pulse_imports'.format(config['server'])
    header = {
        'Accept': 'application/json'
    }
    response = requests.delete(
        path,
        auth=HTTPBasicAuth(
            config['username'],
            config['password']),
        verify=False,
        headers=header,
        proxies=getProxies())
    responseJson = response.json()
    # repsonse puts the deletion task into a queue
    if response.status_code == 404 and responseJson['code'] == 1002:
        print('No need for clean up. pulse_imports reference table doesn\'t exist')
        return

    if 'status' in responseJson:
        verifyPath = 'https://{}/api/reference_data/tables_delete_tasks/{}'.format(config['server'], str(responseJson['id']))
        while (responseJson['status'] != 'COMPLETED'):
            # wait for task to be completed before verifying
            time.sleep(0.5)
            # check if deletion task has completed
            response = requests.get(
                verifyPath,
                auth=HTTPBasicAuth(
                    config['username'],
                    config['password']),
                verify=False,
                headers=header,
                proxies=getProxies())
            
            responseJson = response.json()
            if response.status_code != 200:
                pytest.fail('Failed to query the status of delete reference table request. HTTP Status code: {}. Message: {}.'.format(response.status_code, responseJson['message']))

        print(responseJson['message'])
    else:
        print(response['http_response']['message'])
        pytest.fail('Unable to cleanup reference table.')


def get_templates(hostname, appID, auth):
    url = 'https://' + hostname + "/console/plugins/"+appID+"/app_proxy:nginx/api/template_repos/"
    resp = requests.get(
        url,
        auth=HTTPBasicAuth(
            auth['username'],
            auth['password']),
        verify=False,
        proxies=getProxies())
    assert(resp.status_code == 200)
    json_response = json.loads(resp.text)
    id = ''
    for template in json_response['list']:
        if template['name'] == 'Reference Table':
            id = template['id']
            break
            
    url = 'https://' + hostname + "/console/plugins/"+appID+"/app_proxy:nginx/api/template_repos/"+id+"/source_templates"
    resp = requests.get(
        url,
        auth=HTTPBasicAuth(
            auth['username'],
            auth['password']),
        verify=False, 
        proxies=getProxies())
    assert(resp.status_code == 200)
    json_response = json.loads(resp.text)
    dashboards = []
    json_response = json_response['list']
    for dashboard in json_response:
        dashboards.append(dashboard['name'])
    return dashboards

def get_names(directory):
    fileType = directory + "/*.json"
    fileNames = glob.glob(fileType)
    names = []
    for i in fileNames:
        with open(i) as f:
            data = json.load(f)
            names.append(data['dashboards']['list'][0]['name'])
    return names

def invokeTool(username, password, hostname, directory, clear=False, expectedExitStatus=0):
    proxies = getProxies()
    proxyOpt = ' -x "' + proxies['https'] + '"' if proxies is not None else ''
    clearOpt = ''

    if clear:
        clearOpt = ' -c'
    cmd = 'python "{}" -u "{}" -p "{}" -d "{}" -q "{}"{}{}'.format(toolFullPath, username, password,  directory, hostname, proxyOpt, clearOpt)
    (output, exitstatus) = pexpect.run(cmd, withexitstatus=1)
    if exitstatus != expectedExitStatus:
        print(output)
    assert(exitstatus == expectedExitStatus) 
    return output

def test_dashboard_name_match():
    directory = os.path.join(testdir, "dashboards")
    config = testConfig
    hostname = config['server']
    auth = {
        "username": config['username'],
        "password": config['password']
    }
    deleteTable(config)
    output = invokeTool(config['username'], config['password'], config['server'], directory)
    appID = get_app_ID(hostname, auth)
    dashboards = get_templates(hostname, appID, auth)
    names = get_names(directory)
    assert(names == dashboards)

    # clear the table this time 
    directory = os.path.join(testdir, "dashboards", "Update")
    output = invokeTool(config['username'], config['password'], config['server'], directory, clear=True)
    dashboards = get_templates(hostname, appID, auth)
    names = get_names(directory)
    assert(names == dashboards)

def test_invalid_username():
    directory = os.path.join(testdir, "dashboards")
    config = testConfig
    output = invokeTool('not a valid username', config['password'], config['server'], directory, expectedExitStatus=1)
    assert(output.startswith('Invalid username or password.'))

def test_invalid_password():
    directory = os.path.join(testdir, "dashboards")
    config = testConfig
    output = invokeTool(config['username'], 'bad!$pa$s', config['server'], directory, expectedExitStatus=1)
    assert(output.startswith('Invalid username or password.'))

def test_username_required():
    directory = os.path.join(testdir, "dashboards")
    config = testConfig
    (output, exitstatus) = pexpect.run('python "{}" -p "{}" -d "{}" -q "{}"'.format(toolFullPath, config['password'],  directory, config['server']), withexitstatus=1)
    assert(exitstatus == 2) 
    assert(output.startswith('username is required.'))

def test_directory_required():
    config = testConfig
    (output, exitstatus) = pexpect.run('python "{}" -u "{}" -p "{}" -q "{}"'.format(toolFullPath, config['username'], config['password'], config['server']), withexitstatus=1)
    assert(exitstatus == 2) 
    assert(output.startswith('directory is required.'))
